import Vuex from 'vuex'
import Vue from 'vue'
import createPersistedState from 'vuex-persistedstate'
import {getStoreBuilder} from 'vuex-typex'

import studentPage from './modules/student-page'
import submissionNotes from './modules/submission-notes'
import authentication from './modules/authentication'
import ui from './modules/ui'
import subscriptions from './modules/subscriptions'
import feedbackTable from './modules/feedback_list/feedback-table'
import feedbackSearchOptions from './modules/feedback_list/feedback-search-options'

import './mutations'
import './actions'
import './getters'
import {lastInteraction} from '@/store/plugins/lastInteractionPlugin'
import {
  Exam, Feedback,
  Statistics,
  StudentInfoForListView,
  SubmissionNoType,
  SubmissionType, Tutor
} from '@/models'

Vue.use(Vuex)

export interface RootState {
    lastAppInteraction: number
    examTypes: {[pk: string]: Exam}
    feedback: {[pk: number]: Feedback}
    submissionTypes: {[pk: string]: SubmissionType}
    submissions: {[pk: string]: SubmissionNoType}
    students: {[pk: string]: StudentInfoForListView}
    studentMap: {} // is used to map obfuscated student data back to the original
    statistics: Statistics
    tutors: Array<Tutor>
}

export function initialState (): RootState {
  return {
    lastAppInteraction: Date.now(),
    examTypes: {},
    feedback: {},
    submissionTypes: {},
    submissions: {},
    students: {},
    studentMap: {}, // is used to map obfuscated student data back to the original
    statistics: {
      submissionsPerType: 0,
      submissionsPerStudent: 0,
      currentMeanScore: 0,
      submissionTypeProgress: []
    },
    tutors: []
  }
}

export const persistedStateKey = 'grady'

const store = getStoreBuilder<RootState>().vuexStore({
  strict: process.env.NODE_ENV === 'development',
  modules: {
    authentication,
    studentPage,
    submissionNotes,
    ui,
    subscriptions,
    feedbackTable,
    feedbackSearchOptions
  },
  plugins: [
    createPersistedState({
      key: persistedStateKey,
      storage: window.sessionStorage,
      // authentication.token is manually saved since using it with this plugin caused issues
      // when manually reloading the page
      paths: Object.keys(initialState()).concat(
        ['ui', 'studentPage', 'submissionNotes', 'feedbackSearchOptions', 'subscriptions',
          'authentication.user', 'authentication.jwtTimeDelta',
          'authentication.tokenCreationTime'])
    }),
    lastInteraction],
  state: initialState()
})

export default store
